@@ -1,4 +1,7 @@
 
+from builtins import zip
+from builtins import range
+from builtins import object
 import collections
 import copy
 import re
@@ -14,8 +17,8 @@ class BaseSequence(Spec):
 
     def __init__(self, pos_specs=()):
         self._pos_specs = atom_to_spec_seq(pos_specs)
-        self._setup_from_str_funs()
         self._setup_to_str_funs()
+        self._setup_to_bytes_funs()
         self._unpack_funs = [p.unpack for p in self._pos_specs]
         self._pack_funs = [p.pack for p in self._pos_specs]
         self._struct = struct.Struct(self._struct_fmt)
@@ -26,57 +29,59 @@ class BaseSequence(Spec):
 
     ## unpack
     def unpack(self, text_line):
+        if six.PY3 and isinstance(text_line, str):
+            text_line = text_line.encode()
+        assert isinstance(text_line, bytes)
         try:
             values = self._struct.unpack_from(text_line)
         except struct.error:
             # pad the line out so that struct will take it
             values = self._struct.unpack_from(text_line.ljust(self.width))
-        
         values = [s(v) for s, v in zip(self._unpack_funs, values)]
-        for idx, from_str in self._from_str_funs:
-            values[idx] = from_str(values[idx])
+        #for idx, from_str in self._to_str_funs:
+        #    values[idx] = from_str(values[idx])
         return self._itype(values, self)
 
-    def _setup_from_str_funs(self):
+    def _setup_to_str_funs(self):
         # Functions to call when we convert from a string to a value
-        self._from_str_funs = []
+        self._to_str_funs = []
         for idx, spec in enumerate(self._pos_specs):
             if hasattr(spec, 'from_bytes'):
-                self._from_str_funs.append((idx, spec.from_bytes))
+                self._to_str_funs.append((idx, spec.from_bytes))
 
     @property
     def _struct_fmt(self):
         return ''.join('%ds' % s.width for s in self._pos_specs)
-        
+
     ## Pack
     def pack(self, value):
         # Shortcut
-        if not self._to_str_funs:
-            return ''.join(s(v) for s, v in zip(self._pack_funs, value))
+        #if not self._to_bytes_funs:
+        return b''.join(s(v) for s, v in zip(self._pack_funs, value))
 
-        str_values = list(value)
-        for idx, to_str in self._to_str_funs:
-            str_values[idx] = to_str(value[idx])
-        return ''.join(s(v) for s, v in zip(self._pack_funs, str_values))
+        #str_values = list(value)
+        #for idx, to_str in self._to_bytes_funs:
+        #    str_values[idx] = to_str(value[idx])
+        #return b''.join(s(v) for s, v in zip(self._pack_funs, str_values))
 
     def as_strings(self, value):
         """ Return a builtin NamedTuple that is an 'export' of the value
         into strings
         """
-        if not self._to_str_funs:
+        if not self._to_bytes_funs:
             return self._str_itype(value)
 
         str_value = list(value)
-        for idx, to_str in self._to_str_funs:
+        for idx, to_str in self._to_bytes_funs:
             str_value[idx] = to_str(value[idx])
         return self._str_itype(*str_value)
 
-    def _setup_to_str_funs(self):
+    def _setup_to_bytes_funs(self):
         # Functions to call when we convert from a value to a string
-        self._to_str_funs = []
+        self._to_bytes_funs = []
         for idx, spec in enumerate(self._pos_specs):
             if hasattr(spec, 'to_bytes'):
-                self._to_str_funs.append((idx, spec.to_bytes))
+                self._to_bytes_funs.append((idx, spec.to_bytes))
 
 class _UnconvertedSequenceValueMixIn(object):
     def has_unconverted(self):
@@ -128,7 +133,7 @@ class _TupleValue(tuple, _UnconvertedSequenceValueMixIn):
         return inst
 
     def __init__(self, other, spec, convert_errors=None):
-        tuple.__init__(self, other)
+        tuple.__init__(self)
 
     def __copy__(self):
         rec = _TupleValue(self, self._spec)
@@ -159,8 +164,8 @@ class Array(List):
         self._itype = _ListValue
         self._count = count
         self._pos_specs = [spec] * count
-        self._setup_from_str_funs()
         self._setup_to_str_funs()
+        self._setup_to_bytes_funs()
         self._unpack_funs = [p.unpack for p in self._pos_specs]
         self._pack_funs = [p.pack for p in self._pos_specs]
         self._struct = struct.Struct(self._struct_fmt)
@@ -186,20 +191,20 @@ class _ListValue(list, _UnconvertedSequenceValueMixIn):
             if len(sublist) != len(self) - start:
                 raise TypeError('Cannot change size of list with slice')
             end = len(self) + 1
-        for idx, item in zip(range(start, end), sublist):
+        for idx, item in zip(list(range(start, end)), sublist):
             self.__setitem__(idx, item)
 
     def __setitem__(self, index, values):
         if isinstance(index, slice):
-            indexes = range(*index.indices(len(self)))
+            indexes = list(range(*index.indices(len(self))))
             if len(values) != len(indexes):
                 raise TypeError('Cannot change size of list with slice')
         else:
             indexes = [index]
             values = [values]
-        
+
         for index, str_value in zip(indexes, values):
-            for fun_index, fun in self._spec._from_str_funs:
+            for fun_index, fun in self._spec._to_str_funs:
                 if fun_index == index:
                     value = fun(str_value)
                     break
